@@ -21,10 +21,11 @@ public:
   const char *get_name();
   Stamp *get_stamp();
 private:
-  char *m_name;
+  const char *m_name;
   Stamp *m_stamp;
 };
 
 void set_user_name(const char *name);
+const char *get_user_name();
 
 #endif
