@@ -27,7 +27,8 @@ public:
 private:
   void do_read_file(Storage *st, Ident *id, gzFile fp);
   void add_word(Storage *st, const char *name, Stamp stamp,
-		bool approve, const char *index, const char *word);
+		const char *attr, bool approve, const char *index,
+		const char *word);
   void chomp(char *buf);
 };
 
