@@ -8,14 +8,25 @@
 #include "word.h"
 #include "misc.h"
 
+static void
+do_puts(FILE *fp, gzFile zfp, char *buf)
+{
+  if (fp) {
+    fputs(buf, fp);
+  } else {
+    gzputs(zfp, buf);
+  }
+}
+
 void
-WordSet::add_word(const char *name, Stamp stamp, bool approve,
+WordSet::add_word(const char *name, Stamp stamp,
+		  const char *attr, bool approve,
 		  const char *idx, const char *word)
 {
   Word *w = find_word(idx, word);
-  Attr *a = w->find_attr("add");
+  Attr *a = w->find_attr(attr);
   if (!a) {
-    a = new Attr(w, "add");
+    a = new Attr(w, attr);
     w->m_attrs.push_back(a);
   }
   Sign *s;
@@ -28,9 +39,12 @@ WordSet::add_word(const char *name, Stamp stamp, bool approve,
 Word *
 WordSet::find_word(const char *idx, const char *word)
 {
+  int hash = word_hash(idx, word) % NR_PAGES;
   //
   std::list <Word *>::iterator it;
-  for (it = m_word_array.begin(); it != m_word_array.end(); it++) {
+  for (it = m_word_hash_array[hash].begin();
+       it != m_word_hash_array[hash].end();
+       it++) {
     Word *w = *it;
     if (!strcmp(idx, w->m_index) &&
 	!strcmp(word, w->m_word)) {
@@ -42,11 +56,12 @@ WordSet::find_word(const char *idx, const char *word)
   w->m_index = strdup(idx);
   w->m_word = strdup(word);
   m_word_array.push_back(w);
+  m_word_hash_array[hash].push_back(w);
   return w;
 }
 
 void
-WordSet::write_words(FILE *fp)
+WordSet::write_words(FILE *fp, gzFile zfp)
 {
   std::list<Sign *>::iterator it;
   for (it = m_sign_array.begin(); it != m_sign_array.end(); it++) {
@@ -55,10 +70,12 @@ WordSet::write_words(FILE *fp)
     Word *w = a->m_owner;
     char stamp_buf[10];
     s->m_stamp.get_str(stamp_buf);
-    fprintf(fp, "%s %s %s %s %s %s\n",
+    char buf[1000];
+    sprintf(buf, "%s %s %s %s %s %s\n",
 	    w->m_index, w->m_word,
 	    stamp_buf, s->m_user, a->m_attr,
 	    (s->m_is_ok ? "ok" : "ng"));
+    do_puts(fp, zfp, buf);
   }
 }
 
@@ -105,17 +122,17 @@ Page::read()
 void
 Page::write()
 {
-  FILE *fp = fopen(m_file_name, "w");
-  if (fp) {
-    write_words(fp);
-    fclose(fp);
+  gzFile zfp = gzopen(m_file_name, "wb");
+  if (zfp) {
+    write_words(NULL, zfp);
+    gzclose(zfp);
   }
 }
 
 void
 Page::dump()
 {
-  write_words(stdout);
+  write_words(stdout, NULL);
 }
 
 void
@@ -202,13 +219,14 @@ Storage::update_index()
 }
 
 void
-Storage::add_word(const char *name, Stamp stamp, bool approve,
+Storage::add_word(const char *name, Stamp stamp,
+		  const char *attr, bool approve,
 		  const char *index, const char *word)
 {
   int hash = word_hash(index, word);
   int idx = hash % NR_PAGES;
   Page *p = get_page(idx);
-  p->add_word(name, stamp, approve, index, word);
+  p->add_word(name, stamp, attr, approve, index, word);
 }
 
 Page *
