@@ -2,6 +2,7 @@
 #ifndef _storage_h_included_
 #define _storage_h_included_
 
+#include <zlib.h>
 #include <stdio.h>
 #include <vector>
 #include "word.h"
@@ -12,14 +13,16 @@ class Ident;
 
 class WordSet {
 public:
-  void add_word(const char *name, Stamp stamp, bool approve,
+  void add_word(const char *name, Stamp stamp,
+		const char *attr, bool approve,
 		const char *idx, const char *word);
-  void write_words(FILE *fp);
+  void write_words(FILE *fp, gzFile zfp);
 protected:
   Word *find_word(const char *idx, const char *word);
 public:
   std::list <Word *> m_word_array;
   std::list <Sign *> m_sign_array;
+  std::list <Word *> m_word_hash_array[NR_PAGES];
 };
 
 class Page : public WordSet {
@@ -48,7 +51,8 @@ public:
   void dump();
   void update_index();
 
-  void add_word(const char *name, Stamp stamp, bool approve,
+  void add_word(const char *name, Stamp stamp,
+		const char *attr, bool approve,
 		const char *index, const char *word);
 private:
   Page *get_page(int idx);
