@@ -29,4 +29,13 @@ struct app_win {
   Sign *current_sign;
 };
 
+void build_win(app_win *win);
+void check_cb(GtkWidget *ok_check, app_win *win);
+void word_selection_cb(GtkTreeSelection *selection, app_win *win);
+void attr_selection_cb(GtkTreeSelection *selection, app_win *win);
+void new_attr_cb(GtkWidget *button, app_win *win);
+void save_cb(GtkWidget *button, app_win *win);
+void quit_cb(GtkWidget *button, app_win *win);
+
+
 #endif
